@@ -14,7 +14,7 @@
 # *  limitations under the License.
 # */
 #
-# $Id: BBS.pm 301 2007-03-03 13:51:21Z hikarin $
+# $Id: BBS.pm 907 2007-07-18 14:13:55Z hikarin $
 #
 
 package Img0ch::App::BBS;
@@ -79,14 +79,15 @@ sub new {
 sub run {
     my ($iApp) = @_;
 
-    $iApp->init()    or return $iApp->redirect_error('error');
-    $iApp->_switch() or return 0;
-    $iApp->get_host();
+    $iApp->init()         or return $iApp->redirect_error('error');
+    $iApp->_switch()      or return 0;
+    $iApp->get_host()     or return $iApp->redirect_error('error');
     $iApp->validate()     or return $iApp->redirect_error('error');
     $iApp->is_writeable() or return $iApp->redirect_error('error');
     $iApp->get_date();
     $iApp->post() or return $iApp->redirect_error('error');
-    $iApp->update_subback( undef, 1, 0 );
+    $iApp->update_subback( undef, 1, 0 )
+        or return $iApp->redirect_error('error');
     $iApp->update_index( undef, 1, 0 );
     $iApp->redirect_success();
     return 1;
@@ -109,86 +110,65 @@ sub init {
         return 0;
     }
     $iApp->{__bbs} = $iBBS;
-    $iApp->_plugin( $iBBS, $iConfig, 'bbs.init', 'DisableBBSInitPlugin' )
-        or return 0;
 
     require Img0ch::Setting;
     my $iSetting = Img0ch::Setting->new($iBBS);
     $iSetting->load();
     $iApp->{__setting} = $iSetting;
 
-    my $key = $iRequest->key();
     my $now = $iRequest->now();
-    $key > $now and $key = $now;
-    $iApp->{__now} = $now;
+    $iApp->{__agent} = $iRequest->agent();
+    $iApp->{__now}   = $now;
+
+    my $key = $iRequest->key();
+    if ( !$iConfig->get('DisableThread924') and $key =~ /\A924\d+\z/xms ) {
+        $iApp->{__error} = 'POST_SPECIAL_THREAD';
+        return 0;
+    }
 
     require Img0ch::Thread;
     my $iThread = Img0ch::Thread->new( $iBBS, $key );
-    $iApp->{__agent}    = $iRequest->agent();
-    $iApp->{__key}      = $key;
-    $iApp->{__mkthread} = $iThread->load() ? 0 : 1;
-    $iApp->{__thread}   = $iThread;
+    if ( $iThread->load() ) {
+        $iApp->{__mkthread} = 0;
+        $iApp->{__key}      = $key;
+    }
+    else {
+        $iApp->{__mkthread} = 1;
+        $iApp->{__key}      = $now;
+    }
+    $iApp->{__thread} = $iThread;
+    $iApp->_plugin( $iBBS, $iConfig, 'bbs.init', 'DisableBBSInitPlugin' )
+        or return 0;
     1;
 }
 
 sub get_host {
     my ($iApp)   = @_;
     my $iRequest = $iApp->{_request};
-    my $ip       = $iRequest->ip();
-    my $ip_int   = $iRequest->ip_int();
-    my $host = gethostbyaddr( $ip_int, 2 ) || $ip;
-    my $re = qr/(\d+)\.(\d+)\.(\d+)\.(\d+)/xms;
+    my $host     = $iRequest->get_remote_host();
+    my $re       = qr/\A(\d+)\.(\d+)\.(\d+)\.(\d+)\z/xms;
 
-    if (   $iRequest->get_header('via') =~ /$re/xms
-        || $iRequest->get_header('x-forwarded-for') =~ /$re/xms
-        || $iRequest->get_header('forwarded')       =~ /$re/xms )
+    if (   $iRequest->get_header('via') =~ $re
+        || $iRequest->get_header('x-forwarded-for') =~ $re
+        || $iRequest->get_header('forwarded')       =~ $re )
     {
         my $proxy = gethostbyaddr( pack( 'C4', ( $1, $2, $3, $4 ) ), 2 )
             || join( '.', $1, $2, $3, $4 );
         $host .= '&lt;' . $proxy . '&gt;';
     }
     $iApp->{_host}   = $host;
-    $iApp->{_ip}     = $ip;
-    $iApp->{_ip_int} = $ip_int;
+    $iApp->{_ip}     = $iRequest->ip();
+    $iApp->{_ip_int} = $iRequest->ip_int();
 
     my $mode = $iApp->{__agent};
-    if ( $mode > 0 ) {
-        my $serial = $iApp->{_serial} = [
-            sub {''},
-            sub {
-                my $agent = $_[0]->get_header('user-agent');
-                return $1 if $agent =~ m{/ser(\w{11})\z}xms;
-                return $1 if $agent =~ /;ser(\w{15});/xms;
-                '';
-            },
-            sub {
-                my $agent  = $_[0]->get_header('user-agent');
-                my $serial = '';
-                if ( $agent =~ /\AVodafone/xms ) {
-                    $serial = [ split '/', $agent ]->[4] || '';
-                }
-                else {
-                    $serial = [ split '/', $agent ]->[3] || '';
-                }
-                $serial =~ s/\ASN//xms or return '';
-                $serial =~ s/\s\w*\z//xms;
-                $serial;
-            },
-            sub {
-                my $serial = $_[0]->get_header('x-up-subno');
-                $serial =~ tr/_/-/;
-                $serial =~ /\A(\d{14}-\w\w)/xms ? $1 : '';
-            },
-            sub {''},
-            sub { $_[0]->get_header('x-subscriber-id') },
-            sub {
-                my $agent = $_[0]->get_header('user-agent');
-                $agent =~ /ibisBrowser;.+;\s*(\w+)/xms;
-                $1 || '',;
-            },
-        ]->[$mode]->($iRequest);
-        if ( $serial or $mode == 4 ) {
-            if ( $iApp->is_bbm( $serial, $iRequest->now() ) ) {
+    $mode == 4 and return 1;
+
+    if ( $iRequest->is_mobile_device($mode) ) {
+        my $serial = $iApp->{_serial} = $iRequest->get_device_id($mode);
+        if ($serial) {
+            if ( !$iApp->config()->get('DisableBBMCheck')
+                and $iApp->is_bbm( $serial, $iRequest->now() ) )
+            {
                 $iApp->{__error} = 'DENY_FROM_BBM_LISTED_HOST';
                 return 0;
             }
@@ -208,8 +188,40 @@ sub validate {
     my $iSetting = $iApp->{__setting};
     my $iRequest = $iApp->{_request};
     my $charset  = $iApp->{__kernel}->get_encoding(1);
-    my $name = $iRequest->param_multibyte( 'FROM', $charset );
-    my $mail = $iRequest->param_multibyte( 'mail', $charset );
+    my ( $name, $mail, $comment, $subject );
+
+    if ( $charset eq 'sjis' ) {
+
+        # code from ShiftJIS::String by SADAHIRO tomoyuki
+        my $is_sjis = sub {
+            my $str = shift || '';
+            $str =~ s/[\x00-\x7F\xA1-\xDF]|
+                [\x81-\x9F\xE0-\xFC][\x40-\x7E\x80-\xFC]//gxms;
+            return length $str ? 0 : 1;
+        };
+        $name = $iRequest->param('FROM');
+        if ( !$is_sjis->($name) ) {
+            $name = $iRequest->param_multibyte( 'FROM', $charset );
+        }
+        $mail = $iRequest->param('mail');
+        if ( !$is_sjis->($mail) ) {
+            $mail = $iRequest->param_multibyte( 'mail', $charset );
+        }
+        $comment = $iRequest->param('MESSAGE');
+        if ( !$is_sjis->($comment) ) {
+            $comment = $iRequest->param_multibyte( 'MESSAGE', $charset );
+        }
+        $subject = $iRequest->param('subject');
+        if ( !$is_sjis->($subject) ) {
+            $subject = $iRequest->param_multibyte( 'subject', $charset );
+        }
+    }
+    else {
+        $name    = $iRequest->param_multibyte( 'FROM',    $charset );
+        $mail    = $iRequest->param_multibyte( 'mail',    $charset );
+        $comment = $iRequest->param_multibyte( 'MESSAGE', $charset );
+        $subject = $iRequest->param_multibyte( 'subject', $charset );
+    }
 
     require Img0ch::Cap;
     $mail =~ s/&(?:amp;)?\#(\d+;)/&&$1/gxms;
@@ -230,8 +242,7 @@ sub validate {
 
     $name =~ tr/\r\n\t//d;
     $mail =~ tr/\r\n\t//d;
-    $iApp->escape_chars( \$mail, 0 );
-    $mail =~ /sage/xms and $iApp->{__sage} = 1;
+    ( index( $mail, 'sage' ) >= 0 ) and $iApp->{__sage} = 1;
 
     if ( !$limitless ) {
         if ( $iSetting->get_int('BBS_NAME_COUNT') < length $name ) {
@@ -247,7 +258,9 @@ sub validate {
     }
 
     my $capn;
-    $capn = $iCap->get_name() if $iCap->can_add_cap_name();
+    if ( $iCap->can_add_cap_name() ) {
+        $capn = $iCap->get_name();
+    }
     if ( !$name and !$capn ) {
         $name = $iSetting->get('BBS_NONAME_NAME');
         $iApp->{__param_FROM} = $name;
@@ -275,7 +288,6 @@ sub validate {
         }
     }
 
-    my $comment = $iRequest->param_multibyte( 'MESSAGE', $charset );
     my $len = length $comment;
     if ( $len == 0 ) {
         $iApp->{__error} = 'NO_MESSAGE';
@@ -305,17 +317,18 @@ sub validate {
         $iApp->{__error} = 'TOO_MANY_ANKERS';
         return 0;
     }
-    if ( $iConfig->get('EnableAnnotation') ) {
+    if ( $iSetting->is_checked('BBS_ENABLE_ANNOTATION') ) {
         my @new_comment;
         my $hash = {};
         my $seen = {};
         for my $line ( split "\n", $comment ) {
-            if ($line !~ /\A\@(\w+)\s*[\s=]\s*(.+)\r?\z/xms) {
+            if ( $line !~ /\A\@([\w:]+)\s*[\s=]\s*(.+)\r?\z/xms ) {
                 push @new_comment, $line;
                 next;
             }
             my ( $key, $value ) = ( $1, $2 );
-            if ($seen->{$key}) {
+            $value =~ s/\s+\z//xms;
+            if ( $seen->{$key} ) {
                 push @new_comment, $line;
                 next;
             }
@@ -323,15 +336,15 @@ sub validate {
             $hash->{$key} = $value;
             $line =~ s/\A\@(\w+)[\s=]\s*(.+)\r?\z//xms;
         }
-        %{$iApp->{__annotation}} = %$hash;
+        %{ $iApp->{__annotation} } = %$hash;
         $comment = join "\n", @new_comment;
     }
-    $comment =~ tr/\r\t//d;
     $comment =~ s/\A\s+//xms;
     $comment =~ s/\s+\z//xms;
+    $comment =~ s/\r\n/\n/gxms;
     $comment =~ s/\n/<br>/gxms;
+    $comment =~ tr/\r\t//d;
 
-    my $subject = $iRequest->param_multibyte( 'subject', $charset );
     if ( $iApp->{__mkthread} ) {
         if ( $subject eq '' ) {
             $iApp->{__error} = 'NO_SUBJECT';
@@ -377,7 +390,8 @@ sub is_writeable {
     }
 
     my $resmax = $iApp->{__resmax} = $iConfig->get_int('ResMAX')
-        || $iSetting->get_int('BBS_RES_MAX') || 1000;
+        || $iSetting->get_int('BBS_RES_MAX')
+        || 1000;
     if ( $iApp->{__mkthread} ) {
         if ( $iSetting->is_checked('BBS_THREADCAPONLY')
             and !$iCap->can_create_thread_to_readonly() )
@@ -405,8 +419,9 @@ sub is_writeable {
         my $mkthread = $iApp->{__mkthread};
         my $error    = 0;
         my $max_size = $iSetting->get_int('BBS_DATMAX') * 1024;
-        if ( !$mkthread and !$iThread->can_write(
-            \$error, $max_size, $resmax ) ) {
+        if (    !$mkthread
+            and !$iThread->can_write( \$error, $max_size, $resmax ) )
+        {
             $iApp->{__error} = [
                 'UNKNOWN_ERROR',     'POST_STOPPED_THREAD',
                 'POST_MOVED_THREAD', 'POST_OVER_THREAD',
@@ -445,17 +460,22 @@ sub is_writeable {
 }
 
 sub get_date {
-    my ($iApp) = @_;
+    my ($iApp)   = @_;
     my $iConfig  = $iApp->{_config};
     my $now      = $iApp->{__now};
     my $host     = $iApp->{_host};
-    my $id       = $iApp->get_poster_id(
-         $now, $host, $iConfig->get('Seed'),
-         $iApp->{__bbs}->get_name(), $iApp->{_serial} );
+    my $serial   = $iApp->{_serial};
     my $mode     = '';
     my $iCap     = $iApp->{__cap};
     my $iSetting = $iApp->{__setting};
 
+    my @seed
+        = $iConfig->get('FixIDFromMobile')
+        ? ( $serial || $host )
+        : ( $serial, $host );
+    my $id = $iApp->get_poster_id( $now, @seed, $iConfig->get('Seed'),
+        $iApp->{__bbs}->get_name() );
+
     if ( $iSetting->is_checked('BBS_SLIP') ) {
         $mode = [ '0', 'O', 'O', 'O', 'O', 'Q', 'Q' ]->[ $iApp->{__agent} ];
         $host =~ /p2\.2ch\.net/xms and $mode = 'P';
@@ -511,7 +531,8 @@ sub post {
     my $mode     = $iSetting->get('BBS_MODE');
     if ((      ( $mode eq 'picture' and !$mkthread )
             or ( $mode eq 'news'    and $mkthread )
-            or ( $iSetting->is_checked('BBS_IMG_UPLOAD_THREAD') and $mkthread )
+            or
+            ( $iSetting->is_checked('BBS_IMG_UPLOAD_THREAD') and $mkthread )
         )
         and $iRequest->is_uploadable('file')
         )
@@ -524,15 +545,15 @@ sub post {
     my $iLog = Img0ch::Log->new( $iBBS, $key );
     $iLog->add(
         $count,
-        [   $iApp->{__now},    $iApp->{_ip_int},
+        [   $iApp->{__now},   $iApp->{_ip_int},
             $iApp->{_serial}, $iRequest->get_header('user-agent'),
         ]
     );
     $iLog->save();
 
     $iThread->set(
-        [   $iApp->{__param_FROM}, $iApp->{__param_mail},
-            $iApp->{__param_date}, $iApp->{__param_id},
+        [   $iApp->{__param_FROM},    $iApp->{__param_mail},
+            $iApp->{__param_date},    $iApp->{__param_id},
             $iApp->{__param_MESSAGE}, $iApp->{__param_subject},
         ]
     );
@@ -540,13 +561,13 @@ sub post {
 
     if ( $iApp->{__resmax} <= $count ) {
         require Img0ch::Metadata;
-        my $iMeta = Img0ch::Metadata->new($iBBS);
+        my $iMeta   = Img0ch::Metadata->new($iBBS);
         my $content = $iMeta->thread_end();
-        if (!$content) {
+        if ( !$content ) {
             my $iKernel = $iApp->{__kernel};
             my $path = join '/', $iConfig->get('SystemPath'), '1000.txt';
             local ( *FH, $! );
-            open *FH, "<${path}" ## no critic
+            open *FH, "<${path}"    ## no critic
                 or $iKernel->throw_io_exception($path);
             $content = do { local $/ = undef; <FH> };
             close *FH or $iKernel->throw_io_exception($path);
@@ -563,10 +584,10 @@ sub post {
     my $iSubject = Img0ch::Subject->new($iBBS);
     $iSubject->load();
     if ( !$mkthread and $iApp->{__sage} ) {
-        $iSubject->sage( $iApp->{__key}, $count );
+        $iSubject->sage( $key, $count );
     }
     else {
-        $iSubject->age( $iApp->{__key}, $count, $iApp->{__param_subject} );
+        $iSubject->age( $key, $count, $iApp->{__param_subject} );
     }
     $iSubject->save() or return 0;
     $iApp->{__subject} = $iSubject;
@@ -619,11 +640,10 @@ sub update_subback {
     my $iMeta = Img0ch::Metadata->new($iBBS);
     my $param = {
         ( map { %{$_} } @{ $iApp->{__template_tag_subback} } ),
-        'Banner'   => $iMeta->main_banner(),
-        'Renderer' => ( $iConfig->get('Renderer') || 'read.cgi' ),
-        'RendererSeparator' =>
-            ( $iConfig->get('RendererSeparator') || '/' ),
-        'Threads' => $stack,
+        'Banner'            => $iMeta->main_banner(),
+        'Renderer'          => ( $iConfig->get('Renderer') || 'read.cgi' ),
+        'RendererSeparator' => ( $iConfig->get('RendererSeparator') || '/' ),
+        'Threads'           => $stack,
     };
     $nodisk and return $param;
 
@@ -672,7 +692,7 @@ sub update_index {
         $iThread = Img0ch::Thread->new( $iBBS, 0 );
         $iApp->{__thread} = $iThread;
     }
-    $iThread->enable_xhtml_break( $iConfig->get('EnableXHTMLBreak') ? 1 : 0 );
+    $iConfig->get('EnableXHTMLBreak') and $iThread->enable_xhtml_break();
 
     my $all_threads         = $iSubject->to_array();
     my $num                 = @{$all_threads};
@@ -729,17 +749,17 @@ sub update_index {
     my $meta        = $iMeta->meta();
     my $foot        = $iMeta->foot();
     my $index_param = {
+        'Icons' => [],
         ( map { %{$_} } @{ $iApp->{__template_tag_index} } ),
-        'Banner'   => $iMeta->main_banner(),
-        'FOOT'     => $foot,
-        'HEAD'     => $head,
-        'META'     => $meta,
-        'Renderer' => ( $iConfig->get('Renderer') || 'read.cgi' ),
-        'RendererSeparator' =>
-            ( $iConfig->get('RendererSeparator') || '/' ),
-        'Stored'    => $stored_thread_stack,
-        'SubBanner' => $iMeta->sub_banner(),
-        'Threads'   => $float_thread_stack,
+        'Banner'            => $iMeta->main_banner(),
+        'FOOT'              => $foot,
+        'HEAD'              => $head,
+        'META'              => $meta,
+        'Renderer'          => ( $iConfig->get('Renderer') || 'read.cgi' ),
+        'RendererSeparator' => ( $iConfig->get('RendererSeparator') || '/' ),
+        'Stored'            => $stored_thread_stack,
+        'SubBanner'         => $iMeta->sub_banner(),
+        'Threads'           => $float_thread_stack,
     };
     my $mobile_float_thread_param
         = [ @{$float_thread_stack}, @{$stored_thread_stack}, ];
@@ -747,7 +767,7 @@ sub update_index {
         @{$mobile_float_thread_param},
         0, $iSetting->get_int('BBS_MB_THREAD_NUMBER'),
     );
-    my $mi_param    = {
+    my $mi_param = {
         ( map { %{$_} } @{ $iApp->{__template_tag_mobile_index} } ),
         'Banner'  => $iMeta->mobile_banner(),
         'Count'   => scalar @{$mobile_float_thread_param},
@@ -760,6 +780,7 @@ sub update_index {
     };
     $nodisk and return ( $index_param, $mi_param );
 
+    require Img0ch::Template;
     my $iTemplateIndex = Img0ch::Template->new(
         $iBBS,
         {   file    => 'index',
@@ -788,7 +809,9 @@ sub get_poster_id {
     my @time = localtime($now);
     require Digest::MD5;
     return substr Digest::MD5::md5_base64(
-        @argument, $time[5], $time[4], $time[3] + 31 ), 0, 8;
+        @argument, $time[5], $time[4], $time[3] + 31
+        ),
+        0, 8;
 }
 
 *id = \&get_poster_id;
@@ -796,7 +819,6 @@ sub get_poster_id {
 sub get_date_string {
     my ( $iClass, $now, $msec, $format, $week ) = @_;
 
-    $ENV{'TZ'} = 'Asia/Tokyo';
     my @date = localtime($now);
     $date[5] += 1900;
     $date[4]++;
@@ -834,8 +856,8 @@ sub trip {
         $salt =~ tr/\x3A-\x40\x5B-\x60\x00-\x2D\x7B-\xFF/A-Ga-f./;
         $column = -1 * $column;
         my $t = substr crypt( $key, $salt ), $column;
-        $$name =~ s/&&(\d+;)/&#$1/gxms;
         $$name = $nm ? "${nm} </b>${trip}${t} <b>" : "</b>${trip}${t} <b>";
+        $$name =~ s/&&(\d+;)/&#$1/gxms;
         return 1;
     }
 
@@ -928,31 +950,33 @@ sub is_bbm {
 sub get_plugin_config {
     my ( $iApp, $plugin_id, $name, $merge_level ) = @_;
     my $iKernel = $iApp->{__kernel};
-    my $bbs = $iApp->{__bbs}->get_name();
-    my $key = $iApp->{__key};
-    my $ret = [];
+    my $bbs     = $iApp->{__bbs}->get_id();
+    my $key     = $iApp->{__key};
+    my $ret     = [];
 
     if ( $merge_level & 2 ) {
-        my $config = Img0ch::Plugin->retrive(
-            $plugin_id, $bbs, $key, $iKernel );
-        (( ref $config || '' ) eq 'HASH')
+        my $config
+            = Img0ch::Plugin->retrive( $plugin_id, $bbs, $key, $iKernel );
+        ( ( ref $config || '' ) eq 'HASH' )
             and push @$ret, $config->{$name};
     }
     if ( $merge_level & 1 ) {
-        my $config = Img0ch::Plugin->retrive(
-            $plugin_id, $bbs, undef, $iKernel );
-        (( ref $config || '' ) eq 'HASH')
+        my $config
+            = Img0ch::Plugin->retrive( $plugin_id, $bbs, undef, $iKernel );
+        ( ( ref $config || '' ) eq 'HASH' )
             and push @$ret, $config->{$name};
     }
-    my $config = Img0ch::Plugin->retrive(
-        $plugin_id, undef, undef, $iKernel );
-    (( ref $config || '' ) eq 'HASH')
+    my $config
+        = Img0ch::Plugin->retrive( $plugin_id, undef, undef, $iKernel );
+    ( ( ref $config || '' ) eq 'HASH' )
         and push @$ret, $config->{$name};
     return $ret;
 }
 
 sub is_make_new_thread { $_[0]->{__mkthread} ? 1 : 0 }
 
+*is_thread_builder = \&is_make_new_thread;
+
 sub is_sage { $_[0]->{__sage} ? 1 : 0 }
 
 sub get_name { $_[0]->{__param_FROM} }
@@ -977,7 +1001,13 @@ sub get_ip_int { $_[0]->{_ip_int} }
 
 sub get_remote_host { $_[0]->{_host} }
 
-sub get_annotation { $_[0]->{__annotation}->{$_[1]} || '' }
+sub get_device_id { $_[0]->{_serial} }
+
+sub get_date_and_id { [ $_[0]->{__param_date}, $_[0]->{__param_id} ] }
+
+sub get_annotation { $_[0]->{__annotation}->{ $_[1] } || '' }
+
+sub get_error { $_[0]->{__error} }
 
 sub set_error { $_[0]->{__error} = $_[1]; return; }
 
@@ -1003,7 +1033,7 @@ sub upload { $_[0]->{__upload} }
 
 sub stash {
     my ( $iApp, $key, $value ) = @_;
-    if ( $value ) {
+    if ($value) {
         $iApp->{__stash}->{$key} = $value;
         return;
     }
@@ -1048,6 +1078,16 @@ sub _switch {
     1;
 }
 
+sub trigger_plugin {
+    my ( $iApp, $at, $disable, $iBBS, $iConfig ) = @_;
+    $iBBS    and $iApp->{__bbs}   = $iBBS;
+    $iConfig and $iApp->{_config} = $iConfig;
+    $iBBS    ||= $iApp->{__bbs};
+    $iConfig ||= $iApp->{_config};
+    $at =~ /\Abbs\.\w+\z/xms or return 0;
+    $iApp->_plugin( $iBBS, $iConfig, $at, $disable );
+}
+
 sub _plugin {
     my ( $iApp, $iBBS, $iConfig, $at, $disable ) = @_;
 
@@ -1084,9 +1124,20 @@ sub _upload {
         return 0;
     }
 
+    if ( $iSetting->is_checked('BBS_IMG_REQUIRE_DOWNLOAD_KEY') ) {
+        my $download_key = $iApp->get_annotation('DOWNLOAD')
+            || $iApp->get_annotation('download');
+        if ( !$download_key ) {
+            $iApp->{__error} = 'NO_DLKEY';
+            return 0;
+        }
+        $iUpload->set_download_password($download_key);
+    }
+
     my $remove_key = $iRequest->param('pass');
     if ( !$remove_key
-        and $iSetting->is_checked('BBS_IMG_REQUIRE_REMOVE_KEY') ) {
+        and $iSetting->is_checked('BBS_IMG_REQUIRE_REMOVE_KEY') )
+    {
         $iApp->{__error} = 'UPLOAD_WITHOUT_REMOVE_KEY';
         return 0;
     }
@@ -1104,13 +1155,13 @@ sub _upload {
     my $size = int( ( -s $path || 0 ) / 1024 );
     my $orig = '';
     if ( $iSetting->is_checked('BBS_IMG_DECODE_FILENAME') ) {
-        $orig
-            = ':' . ( pop @{ [ split /[\\\/:]/xms, $filename ] } || 'unknown' );
+        $orig = ':'
+            . ( pop @{ [ split /[\\\/:]/xms, $filename ] } || 'unknown' );
     }
-    $comment = $iUpload->url( $resno, $ext )
-        . " (${size}KB${orig})<br>$comment";
-    my $max = $iSetting->get_int('BBS_IMG_TAG_MAX') || 3;
-    $iUpload->set_tags( \$comment, $max );
+    $comment
+        = $iUpload->url( $resno, $ext ) . " (${size}KB${orig})<br>$comment";
+    my $max = $iSetting->get_int('BBS_IMG_TAG_MAX');
+    $iUpload->set_tags( \$comment, $max, $resno );
     $iApp->{__param_MESSAGE} = $comment;
     $iApp->_plugin( $iBBS, $iConfig, 'bbs.upload.post',
         'DisableBBSPostUploadPlugin' )
@@ -1143,12 +1194,16 @@ sub redirect_success {
     my $iConfig  = $iApp->{_config};
     my $iRequest = $iApp->{_request};
 
-    if ($iRequest->param('js')) {
+    if ( $iRequest->param('js') ) {
         $iRequest->send_http_header( 'text/javascript', 'UTF-8' );
-        $iRequest->print($iRequest->get_json({
-            ok => 1,
-            error => '',
-            errstr => '' }));
+        $iRequest->print(
+            $iRequest->get_json(
+                {   ok     => 1,
+                    error  => '',
+                    errstr => ''
+                }
+            )
+        );
     }
     else {
         require Img0ch::Metadata;
@@ -1159,40 +1214,48 @@ sub redirect_success {
                 setting => $iApp->{__setting},
             }
         );
+        my $is_mobile = $iApp->{_serial} ? 1 : 0;
         $iTemplate->param(
             {   ( map { %{$_} } @{ $iApp->{__template_tag_redirect} } ),
                 'Banner'       => Img0ch::Metadata->new($iBBS)->main_banner(),
+                'IsMobile'     => $is_mobile,
+                'KEY'          => $iApp->{__key},
                 'RedirectWait' => ( $iConfig->get_int('RedirectWait') || 5 ),
+                'Referer'      => $iRequest->get_header('referer'),
             }
         );
         my $charset = $iConfig->get('DefaultCharset');
         $iRequest->send_http_header( 'text/html', $charset );
-        $iTemplate->flush();
+        $iTemplate->flush(
+            $is_mobile
+            ? sub { ${ $_[0] } =~ tr/\n//d; ${ $_[0] } }
+            : undef
+        );
     }
     1;
 }
 
 sub redirect_error {
     my ( $iApp, $template, $cs, $filter, $callback ) = @_;
-    my $iConfig = $iApp->{_config};
-    my $iKernel = $iApp->{__kernel};
-    my $iObject = $iApp->{__bbs} || $iKernel;
+    my $iConfig  = $iApp->{_config};
+    my $iKernel  = $iApp->{__kernel};
+    my $iObject  = $iApp->{__bbs} || $iKernel;
     my $iRequest = $iApp->{_request};
 
     require Img0ch::Error;
     my $key = $iApp->{__key} || $iRequest->key();
     my $iError = Img0ch::Error->new( $iObject, $key, $iApp->{_ip_int} );
-    my $error  = $iApp->{__error} || 'UNKNOWN_ERROR';
-    my $long   = $iError->get($error);
+    my $error = $iApp->{__error} || 'UNKNOWN_ERROR';
+    my $long = $iError->get($error);
 
     $iError->record();
-    while ( my ( $subs, $value ) = each %{$iApp->{__error_args}} ) {
+    while ( my ( $subs, $value ) = each %{ $iApp->{__error_args} } ) {
         $long =~ s/{!$subs!}/$value/gxms;
     }
 
     my $charset = $iKernel->get_encoding(1);
-    my $ip   = $iApp->{_host} || $iApp->{_ip} || $iRequest->ip();
-    my $name = $iApp->{__param_FROM}
+    my $ip      = $iApp->{_host} || $iApp->{_ip} || $iRequest->ip();
+    my $name    = $iApp->{__param_FROM}
         || $iRequest->param_multibyte( 'FROM', $charset );
     my $mail = $iApp->{__param_mail}
         || $iRequest->param_multibyte( 'mail', $charset );
@@ -1200,29 +1263,37 @@ sub redirect_error {
         || $iRequest->param_multibyte( 'MESSAGE', $charset );
     $callback and $callback->( $iApp, $long );
 
-    if ($iRequest->param('js')) {
+    if ( $iRequest->param('js') ) {
         require Unicode::Japanese;
         $iRequest->send_http_header( 'text/javascript', 'UTF-8' );
         $long =~ tr/\r//d;
         my $icode = Unicode::Japanese->getcode($long);
-        $iRequest->print($iRequest->get_json({
-            ok => 0,
-            error => $error,
-            errstr => Unicode::Japanese->new( $long, $icode )->get() }));
+        $iRequest->print(
+            $iRequest->get_json(
+                {   ok     => 0,
+                    error  => $error,
+                    errstr => Unicode::Japanese->new( $long, $icode )->get()
+                }
+            )
+        );
     }
     else {
         require Img0ch::Template;
         my $iTemplate = Img0ch::Template->new(
             $iObject,
-            {   file    => ( $template || 'error' ),
+            {   file => ( $template || 'error' ),
                 request => $iRequest,
                 setting => $iApp->{__setting},
             }
         );
+        my $is_mobile = $iRequest->is_mobile_device();
+        $is_mobile and $filter = sub { ${ $_[0] } =~ tr/\n//d; ${ $_[0] } };
         $iTemplate->param(
             {   ( map { %{$_} } @{ $iApp->{__template_tag_error} } ),
                 'Flag'          => 1,
                 'Host'          => $ip,
+                'IsMobile'      => $is_mobile,
+                'KEY'           => $iApp->{__key},
                 'LongErrorStr'  => $long,
                 'Name'          => $name,
                 'Mail'          => $mail,
@@ -1917,6 +1988,38 @@ $host
 
 現在の投稿元(リモートホスト)の文字列を返します。
 
+=head2 get_device_id
+
+=over 4
+
+=item Arguments
+
+none
+
+=item Return Value
+
+$device_id
+
+=back
+
+現在の投稿元が保有する端末固有の識別IDを返します。
+
+=head2 get_date_and_id
+
+=over 4
+
+=item Arguments
+
+none
+
+=item Return Value
+
+[ $date, id ]
+
+=back
+
+現在の時刻とID(掲示板設定に依存する)が入った配列のリファレンスを返します。
+
 =head2 get_annotation
 
 =over 4
@@ -1934,9 +2037,30 @@ $annotation_value
 本文に付けられたアノテーションに対応する値を取得します。
 対応する値が存在しない場合はナルストリングが返されます。
 
-img0ch-config.cgiにI<EnableAnnotation>が入っていない場合
+  # 本文中
+  @ANNOTATION foobar
+  # コード上
+  $iApp->get_annotation('ANNOTATION'); # foobar
+
+掲示板設定のI<BBS_ENABLE_ANNOTATION>がchecked(有効)でない場合
 常にこの関数はナルストリングが返されます。
 
+=head2 get_error
+
+=over 4
+
+=item Arguments
+
+none
+
+=item Return Value
+
+$error
+
+=back
+
+現在のエラー内容を返します。
+
 =head2 set_error
 
 =over 4
@@ -2107,6 +2231,22 @@ $iUpload (Img0ch::Upload)
 I<Img0ch::Upload>オブジェクトを返します。
 アップロードされることが分かっている場合に利用可能です。
 
+=head2 trigger_plugin
+
+=over 4
+
+=item Arguments
+
+$at, $disable, $iBBS?, $iConfig?
+
+=item Return Value
+
+1 or 0
+
+=back
+
+bbsに関連するプラグインを実行させます。
+
 =head2 stash
 
 =over 4
@@ -2246,6 +2386,8 @@ none
 書き込み成功画面用のユーザー定義タグを追加します。
 値は必ずハッシュリファレンスを渡す必要があります。
 
-hkrn
+=head1 AUTHOR
+
+hkrn E<lt>hikarin@users.sourceforge.jpE<gt>
 
 =cut
