@@ -14,7 +14,7 @@
 # *  limitations under the License.
 # */
 #
-# $Id: UFM.pm 256 2007-02-17 14:37:05Z hikarin $
+# $Id: UFM.pm 809 2007-06-15 15:02:57Z hikarin $
 #
 
 package Img0ch::App::UFM;
@@ -24,20 +24,21 @@ use strict;
 sub new {
     my ( $iClass, $iKernel, @args ) = @_;
     bless {
-        __bbs => undef,
-        __config => $iKernel->get_config(),
-        __kernel => $iKernel,
-        __resno => 0,
+        __bbs     => undef,
+        __config  => $iKernel->get_config(),
+        __kernel  => $iKernel,
+        __key     => 0,
+        __resno   => 0,
         __request => Img0ch::Request->new(@args),
         __setting => undef,
-        }, $iClass;
+    }, $iClass;
 }
 
 sub run {
     my ($iApp) = @_;
     $iApp->init() or return $iApp->redirect_error('init');
 
-    my $key = $iApp->{__key};
+    my $key   = $iApp->{__key};
     my $resno = $iApp->{__resno};
     my ( $result, $iTemplate, $ctype, $body );
     if ( $key and $resno ) {
@@ -45,7 +46,7 @@ sub run {
             = $iApp->process( $key, $resno );
     }
     else {
-        ( $result, $iTemplate, $ctype, $body ) =  $iApp->render();
+        ( $result, $iTemplate, $ctype, $body ) = $iApp->render();
     }
 
     $result or return $iApp->redirect_error('remove');
@@ -58,10 +59,10 @@ sub run {
 sub init {
     my ($iApp) = @_;
     my $iRequest = $iApp->{__request};
-    $iRequest->init($iApp->{__config});
+    $iRequest->init( $iApp->{__config} );
 
-    my $bbs = $iRequest->bbs();
-    my $key = $iRequest->param_int('key');
+    my $bbs   = $iRequest->bbs();
+    my $key   = $iRequest->param_int('key');
     my $resno = $iRequest->param_int('resno');
     if ( my $path_info = $iRequest->path_info() ) {
         my @path = split '/', $path_info;
@@ -84,9 +85,9 @@ sub init {
     my $iSetting = Img0ch::Setting->new($iBBS);
     $iSetting->load();
 
-    $iApp->{__bbs} = $iBBS;
-    $iApp->{__key} = $key;
-    $iApp->{__resno} = $resno;
+    $iApp->{__bbs}     = $iBBS;
+    $iApp->{__key}     = $key;
+    $iApp->{__resno}   = $resno;
     $iApp->{__setting} = $iSetting;
 
     return 1;
@@ -94,7 +95,7 @@ sub init {
 
 sub process {
     my ( $iApp, $key, $resno ) = @_;
-    my $iBBS = $iApp->{__bbs};
+    my $iBBS     = $iApp->{__bbs};
     my $iRequest = $iApp->{__request};
 
     require Img0ch::Upload;
@@ -103,48 +104,40 @@ sub process {
     my $process = {};
     if ($pass) {
         my $type = $iRequest->param('type');
-        my $js = {};
-        if ( $type eq 'remove'
-            and $iUpload->can_remove( $resno, $pass ) ) {
+        my $js   = {};
+        if (    $type eq 'remove'
+            and $iUpload->can_remove( $resno, $pass ) )
+        {
             $iUpload->remove($resno);
             $iUpload->save();
             $process->{removed} = 1;
-            $js->{ok} = 1;
+            $js->{ok}           = 1;
         }
         elsif ( $type eq 'download'
-            and $iUpload->can_download( $resno, $pass ) ) {
+            and $iUpload->can_download( $resno, $pass ) )
+        {
             my $iKernel = $iApp->{__kernel};
-            my ($ext) = @{ $iUpload->get($resno) };
-            my $path = $iUpload->path( $resno, $ext );
+            my ($ext)   = @{ $iUpload->get($resno) };
+            my $path    = $iUpload->path( $resno, $ext );
             local ( *FH, $! );
-            open *FH, "<${path}" ## no critic
+            open *FH, "<${path}"    ## no critic
                 or $iKernel->throw_io_exception($path);
             binmode *FH;
             my $buffer = do { local $/ = undef; <FH> };
             close *FH or $iKernel->throw_io_exception($path);
-            $iRequest->set_header(
-                'content-disposition',
+            $iRequest->set_header( 'content-disposition',
                 'attachment; filename="' . "${resno}.${ext}" . '"' );
             $iRequest->set_header( 'content-length', length($buffer) );
-            return (
-                1,
-                undef,
-                'application/octet-stream',
-                \$buffer,
-                );
+            return ( 1, undef, 'application/octet-stream', \$buffer, );
         }
         elsif ( $type eq 'confirm' ) {
-            $js->{remove} = $iUpload->can_remove( $resno, $pass );
+            $js->{remove}   = $iUpload->can_remove( $resno,   $pass );
             $js->{download} = $iUpload->can_download( $resno, $pass );
-            $process->{ok} = 1;
+            $process->{ok}  = 1;
         }
-        if ($iRequest->param('js')) {
-            return (
-                ( $process->{ok} || $process->{removed} || 0 ),
-                undef,
-                'text/javascript',
-                $iRequest->get_json($js)
-                );
+        if ( $iRequest->param('js') ) {
+            return ( ( $process->{ok} || $process->{removed} || 0 ),
+                undef, 'text/javascript', $iRequest->get_json($js) );
         }
     }
 
@@ -153,9 +146,9 @@ sub process {
 
 sub render {
     my ( $iApp, $process, $key_param, $resno_param, $pass_param ) = @_;
-    my $iBBS = $iApp->{__bbs};
+    my $iBBS     = $iApp->{__bbs};
     my $iRequest = $iApp->{__request};
-    $key_param ||= $iApp->{__key};
+    $key_param   ||= $iApp->{__key};
     $resno_param ||= $iApp->{__resno};
 
     require Img0ch::Subject;
@@ -179,7 +172,7 @@ sub render {
 
     require Img0ch::Metadata;
     require Img0ch::Template;
-    my $iMeta = Img0ch::Metadata->new($iBBS);
+    my $iMeta     = Img0ch::Metadata->new($iBBS);
     my $iTemplate = Img0ch::Template->new(
         $iBBS,
         {   file    => 'ufm',
@@ -187,56 +180,54 @@ sub render {
             setting => $iApp->{__setting},
         }
     );
-    my $removed => $process->{removed};
-    $iTemplate->param({
-        Banner  => $iMeta->main_banner(),
-        KEY => $key_param,
-        Password => $pass_param,
-        Removed => $removed,
-        Failed => ( $pass_param and !$removed ),
-        Resno => $resno_param,
-        Threads => $stack,
-        });
+    my $removed = $process->{removed};
+    $iTemplate->param(
+        {   Banner   => $iMeta->main_banner(),
+            KEY      => $key_param,
+            META     => $iMeta->meta(),
+            Password => $pass_param,
+            Removed  => $removed,
+            Failed   => ( $pass_param and !$removed ),
+            Resno    => $resno_param,
+            Threads  => $stack,
+        }
+    );
 
-    return (
-        1,
-        $iTemplate,
-        'text/html',
-        undef,
-        );
+    return ( 1, $iTemplate, 'text/html', undef, );
 }
 
 sub redirect_error {
     my ( $iApp, $reason ) = @_;
-    my $iKernel = $iApp->{__kernel};
+    my $iKernel  = $iApp->{__kernel};
     my $iRequest = $iApp->{__request};
-    my $key = $iRequest->key();
-    my $ip = $iRequest->ip_int();
+    my $key      = $iRequest->key();
+    my $ip       = $iRequest->ip_int();
 
     require Img0ch::Error;
     my $iError = Img0ch::Error->new( $iKernel, $key, $ip );
     my $long = $iError->get( $iApp->{__error} || 'INVALID_POST' );
     $iError->record();
 
-    if ($iRequest->param('js')) {
+    if ( $iRequest->param('js') ) {
         $iRequest->send_http_header( 'text/javascript', 'UTF-8' );
-        $iRequest->print($iRequest->get_json({
-            ok => 0, error => $reason } ));
+        $iRequest->print(
+            $iRequest->get_json( { ok => 0, error => $reason } ) );
     }
     else {
         require Img0ch::Template;
         my $iTemplate = Img0ch::Template->new(
             ( $iApp->{__bbs} || $iKernel ),
-            {   file    => ( 'error' ),
+            {   file    => ('error'),
                 request => $iRequest,
                 setting => $iApp->{__setting},
             }
         );
-        $iTemplate->param({
-            Banner => '',
-            Flag => 0,
-            LongErrorStr => $long,
-            });
+        $iTemplate->param(
+            {   Banner       => '',
+                Flag         => 0,
+                LongErrorStr => $long,
+            }
+        );
         my $iConfig = $iApp->{__config};
         my $charset = $iConfig->get('DefaultCharset');
         $iRequest->send_http_header( 'text/html', $charset );
