@@ -14,7 +14,7 @@
 # *  limitations under the License.
 # */
 #
-# $Id: Upload.pm 164 2007-02-02 14:52:06Z hikarin $
+# $Id: Upload.pm 363 2007-03-16 15:11:10Z hikarin $
 #
 
 package Img0ch::Compat::Upload;
@@ -23,8 +23,8 @@ use strict;
 
 sub load {
     my ( $iRepos, $iBBS, $key ) = @_;
-    my $base    = $iBBS->path($key);
-    my $path    = join '/', $base, 'upload.cgi';
+    my $base = $iBBS->path($key);
+    my $path = join '/', $base, 'upload.cgi';
     -r $path or return 0;
 
     my $iKernel = $iBBS->get_kernel();
@@ -33,26 +33,27 @@ sub load {
     my $i       = $iRepos->get("${prefix}._");
 
     local ( $!, *FH );
-    open *FH, "<${path}" ## no critic
+    open *FH, "<${path}"    ## no critic
         or $iKernel->throw_io_exception($path);
     while ( my $line = <FH> ) {
         chomp $line;
-        my ( $resno, $extension, $width, $height, $hash,
-            $rmkey, $suspend, $ip, $time ) = split ',', $line;
+        my ($resno, $extension, $width, $height, $hash,
+            $rmkey, $suspend,   $ip,    $time
+        ) = split ',', $line;
         my $prefix_resno = join '.', $prefix, $resno;
         if ( !$iRepos->get("${prefix_resno}.extension") ) {
             $iRepos->set( "${prefix_resno}.extension", $extension );
-            $iRepos->set( "${prefix_resno}.width", $width );
-            $iRepos->set( "${prefix_resno}.height", $height );
-            $iRepos->set( "${prefix_resno}.hash", $hash );
+            $iRepos->set( "${prefix_resno}.width",     $width );
+            $iRepos->set( "${prefix_resno}.height",    $height );
+            $iRepos->set( "${prefix_resno}.hash",      $hash );
             my $thumbnail = "${path}/t${resno}.${extension}";
             if ( -r $thumbnail ) {
                 my ( $thumb_width, $thumb_height )
                     = Img0ch::Upload->get_dimensions($thumbnail);
-                $iRepos->set(
-                    "${prefix_resno}.width.thumbnail", $thumb_width );
-                $iRepos->set(
-                    "${prefix_resno}.height.thumbnail", $thumb_height );
+                $iRepos->set( "${prefix_resno}.width.thumbnail",
+                    $thumb_width );
+                $iRepos->set( "${prefix_resno}.height.thumbnail",
+                    $thumb_height );
             }
             $i++;
         }
