@@ -14,7 +14,7 @@
 # *  limitations under the License.
 # */
 #
-# $Id: Log.pm 346 2007-03-14 14:54:42Z hikarin $
+# $Id: Log.pm 915 2007-07-19 15:02:38Z hikarin $
 #
 
 package Img0ch::Log;
@@ -24,25 +24,14 @@ use strict;
 sub new {
     my ( $iClass, $iBBS, $key ) = @_;
     my $iKernel = $iBBS->get_kernel();
-
-    my ( undef, undef, undef, $day, $month, $year ) = localtime($key);
-    $year  += 1900;
-    $month += 1;
-
-    my $repos = $iBBS->get_repos_path( $year, $month, $day, $key, 'log' );
-    my $repos_dir = $repos;
-    $repos_dir =~ s{/log\..*\z}{}xms;
-    if ( !-d $repos_dir ) {
-        require File::Path;
-        File::Path::mkpath($repos_dir)
-            or $iKernel->throw_io_exception($repos_dir);
-    }
+    my $repos_path
+        = Img0ch::Maple::create_repository_subdirs( $key, 'log', $iBBS );
 
     bless {
         _bbs  => $iBBS->get_name(),
         _key  => $key,
         __log => [],
-        _rs   => $iKernel->get_repos_nocache($repos),
+        _rs   => $iKernel->get_repos_nocache($repos_path),
     }, $iClass;
 }
 
@@ -83,7 +72,6 @@ sub get {
     my $date   = $iRepos->get("${prefix}.date");
     if ($date) {
         my $ip     = $iRepos->get("${prefix}.ip");
-        my $date   = $iRepos->get("${prefix}.date");
         my $serial = $iRepos->get("${prefix}.serial");
         my $agent  = $iRepos->get_binary("${prefix}.agent");
         $ip = pack 'C*', split( '\.', $ip );
