@@ -14,15 +14,15 @@
 # *  limitations under the License.
 # */
 #
-# $Id: Maple.pm 210 2007-02-10 12:53:52Z hikarin $
+# $Id: Maple.pm 917 2007-07-20 15:34:41Z hikarin $
 #
 
 package Img0ch::Maple;
 
 use strict;
 
-$Img0ch::Maple::RCS_ID   = '$Id: Maple.pm 210 2007-02-10 12:53:52Z hikarin $';
-$Img0ch::Maple::VERSION  = '2.1.5';
+$Img0ch::Maple::RCS_ID   = '$Id: Maple.pm 917 2007-07-20 15:34:41Z hikarin $';
+$Img0ch::Maple::VERSION  = '2.1.15';
 $Img0ch::Maple::revision = do {
     my @rv = $Img0ch::Maple::RCS_ID =~ /(\d+)/xms;
     $rv[0];
@@ -33,6 +33,7 @@ sub new {
     my $repos_type
         = $iClass->get_repos_type( $iConfig->get('RepositoryType') );
     require "Img0ch/Maple/${repos_type}.pm";    ## no critic
+    $ENV{TZ} ||= $iConfig->get('TimeZone') || 'JST-9';
 
     bless {
         __cache  => {},
@@ -88,7 +89,6 @@ sub throw_io_exception {
 
 sub throw_exception {
     my ( $iClass, @err ) = @_;
-
     die @err, "\n", _trace()->as_string(), "\n";
 }
 
@@ -126,27 +126,92 @@ sub get_repos_path {
     return $iConfig->get('RepositoryRoot') . '/' . $path . '.' . $ext;
 }
 
-sub get_repos_type {
-    my ( $iClass, $repos_type ) = @_;
-    return {
-        'Archive'    => 'Archive',
-        'BDB'        => 'BDB',
-        'BerkeleyDB' => 'BDB',
-        'CDB'        => 'CDB',
-        'CDB_File'   => 'CDB',
-        'Compressed' => 'Archive',
-        'DBMDeep'    => 'DD',
-        'DD'         => 'DD',
-        'Depot'      => 'Qdepot',
-        'FlatFile'   => 'Simple',
-        'QDBM'       => 'Qdepot',
-        'Qdepot'     => 'Qdepot',
-        'Qvilla'     => 'Qvilla',
-        'Simple'     => 'Simple',
-        'Storable'   => 'Storable',
-        'Villa'      => 'Qvilla',
-        }->{$repos_type}
-        || 'Simple';
+sub start_transaction {
+    my ($iMaple) = @_;
+    return;
+}
+
+sub commit_transaction {
+    my ($iMaple) = @_;
+    return;
+}
+
+sub rollback_transaction {
+    my ($iMaple) = @_;
+    return;
+}
+
+sub create_repository_subdirs {
+    my ( $time, $filename, $iObject ) = @_;
+    my ( undef, undef, undef, $day, $month, $year ) = localtime($time);
+
+    $year  += 1900;
+    $month += 1;
+    my $repos_file_path
+        = $iObject->get_repos_path( join '/', $year, $month, $day, $time,
+        $filename );
+    my $repos_file_dir = $repos_file_path;
+    $repos_file_dir =~ s{/$filename.\w+\z}{}xms;
+
+    if ( !-d $repos_file_dir ) {
+        require File::Path;
+        my $iKernel
+            = $iObject->isa('Img0ch::BBS')
+            ? $iObject->get_kernel()
+            : $iObject;
+        File::Path::mkpath($repos_file_dir)
+            or $iKernel->throw_io_exception($repos_file_dir);
+    }
+
+    return $repos_file_path;
+}
+
+my $repository_type = {
+    'Archive'    => 'Archive',
+    'BDB'        => 'BDB',
+    'BerkeleyDB' => 'BDB',
+    'CDB'        => 'CDB',
+    'CDB_File'   => 'CDB',
+    'Compressed' => 'Archive',
+    'DBMDeep'    => 'DD',
+    'DD'         => 'DD',
+    'Depot'      => 'Qdepot',
+    'FlatFile'   => 'Simple',
+    'QDBM'       => 'Qdepot',
+    'Qdepot'     => 'Qdepot',
+    'Qvilla'     => 'Qvilla',
+    'Simple'     => 'Simple',
+    'Storable'   => 'Storable',
+    'Villa'      => 'Qvilla',
+};
+
+sub get_repos_type { $repository_type->{ $_[1] } || 'Simple' }
+
+my $module_by_script = {
+    'bbs.cgi'         => 'Img0ch::App::BBS',
+    'img0ch-note.cgi' => 'Img0ch::App::Note',
+    'img0ch-sf.cgi'   => 'Img0ch::App::Search::File',
+    'img0ch-st.cgi'   => 'Img0ch::App::Search::Text',
+    'img0ch-ufm.cgi'  => 'Img0ch::App::UFM',
+    'm.cgi'           => 'Img0ch::App::MailPost',
+    'oekaki.cgi'      => 'Img0ch::Compat::App::Oekaki',
+    'r.cgi'           => 'Img0ch::App::Mobile',
+    'read.cgi'        => 'Img0ch::App::Read',
+    'zeromin2.cgi'    => 'Zeromin2::App',
+};
+
+sub get_application_modules { [ values %{$module_by_script} ] }
+
+sub get_module_by_script { $module_by_script->{ $_[1] } || '' }
+
+sub load_application_modules {
+    my @modules = values %{$module_by_script};
+    for my $app (@modules) {
+        $app .= '.pm';
+        $app =~ s{::}{/}gxms;
+        require $app;
+    }
+    return;
 }
 
 sub _trace {
@@ -345,6 +410,54 @@ $number
 文字列を強制的に数値に変換します。文字列が全て数字の場合は
 その値を返し、文字が含まれている場合は0を返します。
 
+=head2 get_application_modules
+
+=over 4
+
+=item Arguments
+
+none
+
+=item Return Value
+
+$array_reference_of_all_application_modules
+
+=back
+
+Img0ch::App::*のモジュールのパッケージ名を配列リファレンスとして返します。
+
+=head2 get_module_by_script
+
+=over 4
+
+=item Arguments
+
+$cgi_script_name
+
+=item Return Value
+
+$application_module_name
+
+=back
+
+I<$cgi_script_name>から対応するアプリケーションモジュールのパッケージ名を返します。
+
+=head2 load_application_modules
+
+=over 4
+
+=item Arguments
+
+none
+
+=item Return Value
+
+none
+
+=back
+
+Img0ch::App::*のモジュールを全て読み込みます。
+
 =head2 get_repos
 
 =over 4
@@ -397,6 +510,7 @@ I<RepositoryType>からモジュールを読み込めるようにする値を返
 一致する値が存在しない場合I<Simple(Img0ch::Maple::Simple)>を返します。
 この関数はバージョンが2.1.x-3.xでのみ使用可能です。
 
+
 =head1 AUTHOR
 
 hkrn E<lt>hikarin@users.sourceforge.jpE<gt>
