@@ -14,7 +14,7 @@
 # *  limitations under the License.
 # */
 #
-# $Id: Upload.pm 718 2007-05-29 13:19:56Z hikarin $
+# $Id: Upload.pm 915 2007-07-19 15:02:38Z hikarin $
 #
 
 package Img0ch::Upload;
@@ -30,20 +30,9 @@ sub new {
     my $bbs       = $iBBS->get_name();
 
     my $repos_upload = '';
-    if ($key) {
-        my ( undef, undef, undef, $day, $month, $year ) = localtime($key);
-        $year  += 1900;
-        $month += 1;
-        $repos_upload
-            = $iBBS->get_repos_path( $year, $month, $day, $key, 'upload' );
-        my $repos_upload_dir = $repos_upload;
-        $repos_upload_dir =~ s{/upload.\w+\z}{}xms;
-        if ( !-d $repos_upload_dir ) {
-            require File::Path;
-            File::Path::mkpath($repos_upload_dir)
-                or $iKernel->throw_io_exception($repos_upload_dir);
-        }
-    }
+    $key
+        and $repos_upload
+        = Img0ch::Maple::create_repository_subdirs( $key, 'upload', $iBBS );
 
     my ( $iReposUpload, $count ) = ( '', 0 );
     if ($repos_upload) {
@@ -133,6 +122,7 @@ sub save {
 
     @{ $iUpload->{__upload} } = ();
     $iUpload->{__path} = $path;
+    $iUpload->{__url} = $iUpload->url( $resno, $ext );
     $iReposUpload->save();
     return 1;
 }
@@ -171,7 +161,9 @@ sub update_csv {
         rename $temp, $path or $iKernel->throw_io_exception($path);
     };
     if ( my $exception = $@ ) {
-        unlink $temp or $iKernel->throw_io_exception($temp);
+        if ( -e $temp ) {
+            unlink $temp or $iKernel->throw_io_exception($temp);
+        }
         die $exception, "\n";
     }
 
@@ -180,7 +172,7 @@ sub update_csv {
 
 sub get {
     my ( $iUpload, $resno ) = @_;
-    my $prefix       = join '.', $iUpload->{__prefix}, $resno;
+    my $prefix = join '.', $iUpload->{__prefix}, $resno;
     my $iReposUpload = $iUpload->{_rs};
 
     if ( my $hash = $iReposUpload->get("${prefix}.hash") ) {
@@ -196,7 +188,7 @@ sub get {
 
 sub get_thumbnail_size {
     my ( $iUpload, $resno, $ref_width, $ref_height ) = @_;
-    my $prefix       = join '.', $iUpload->{__prefix}, $resno;
+    my $prefix = join '.', $iUpload->{__prefix}, $resno;
     my $iReposUpload = $iUpload->{_rs};
 
     if ( my $width = $iReposUpload->get_int("${prefix}.width.thumbnail") ) {
@@ -234,8 +226,8 @@ sub set_key {
             or $iKernel->throw_io_exception($repos_upload_dir);
     }
 
-    my $prefix           = "I:U:I.${bbs}.${key}";
-    my $iReposUpload     = $iKernel->get_repos($repos_upload);
+    my $prefix       = "I:U:I.${bbs}.${key}";
+    my $iReposUpload = $iKernel->get_repos($repos_upload);
     $iUpload->{_count}   = $iReposUpload->get_int("${prefix}._");
     $iUpload->{_path}    = $iBBS->path( 'img', $key );
     $iUpload->{_rs}      = $iReposUpload;
@@ -251,14 +243,14 @@ sub set_remove_password { $_[0]->{__rmpass} = $_[1]; return; }
 sub have_download_password {
     my ( $iUpload, $resno ) = @_;
     my $iReposUpload = $iUpload->{_rs};
-    return $iReposUpload->get(
-        join '.', $iUpload->{__prefix}, $resno, 'download.pass') ? 1 : 0;
+    return $iReposUpload->get( join '.', $iUpload->{__prefix}, $resno,
+        'download.pass' ) ? 1 : 0;
 }
 
 sub set_thumbnail_size {
     my ( $iUpload, $resno, $width, $height ) = @_;
     my $iReposUpload = $iUpload->{_rs};
-    my $prefix       = join '.', $iUpload->{__prefix}, $resno;
+    my $prefix = join '.', $iUpload->{__prefix}, $resno;
     $iReposUpload->set( "${prefix}.width.thumbnail",  $width );
     $iReposUpload->set( "${prefix}.height.thumbnail", $height );
     return;
@@ -286,7 +278,7 @@ sub can_remove { shift->_can( 'remove', @_ ) }
 
 sub _can {
     my ( $iUpload, $type, $resno, $pass ) = @_;
-    my $key_by_type  = join '.', $iUpload->{__prefix}, $resno, $type, 'pass';
+    my $key_by_type = join '.', $iUpload->{__prefix}, $resno, $type, 'pass';
     my $iReposUpload = $iUpload->{_rs};
 
     if ( my $pass_md5 = $iReposUpload->get($key_by_type) ) {
@@ -318,8 +310,11 @@ sub get_dimensions {
     my ( $iClass, $path ) = @_;
     my ( $width, $height ) = ( 0, 0 );
 
-    if ( $path =~ /\.jpg\z/xms and ( defined $INC{'Image/Epeg.pm'}
-        or eval 'use Image::Epeg qw(); 1' ) ) {
+    if ($path =~ /\.jpg\z/xms
+        and ( defined $INC{'Image/Epeg.pm'}
+            or eval 'use Image::Epeg qw(); 1' )
+        )
+    {
         my $epeg = Image::Epeg->new($path);
         if ($epeg) {
             $width  = $epeg->get_width();
@@ -350,7 +345,8 @@ sub get_dimensions {
         }
     }
     elsif ( defined $INC{'Image/Magick.pm'}
-        or eval 'use Image::Magick qw(); 1' ) {
+        or eval 'use Image::Magick qw(); 1' )
+    {
         my $image = Image::Magick->new;
         ( $width, $height ) = $image->Ping($path);
     }
@@ -368,6 +364,8 @@ sub get_dimensions {
 
 sub get_uploaded_file_path { $_[0]->{__path} }
 
+sub get_uploaded_file_url { $_[0]->{__url} }
+
 sub get_tags {
     my ( $iUpload, $tag ) = @_;
     my $iReposTag  = $iUpload->{_tag};
@@ -518,7 +516,7 @@ sub remove {
     $iReposUpload->remove("${prefix}.width.thumbnail");
     $iReposUpload->remove("${prefix}.height.thumbnail");
     $iReposUpload->remove("${prefix}.download.pass");
-    $iReposUpload->set( "${prefix}.hash",        $iUpload->removed_mark() );
+    $iReposUpload->set( "${prefix}.hash",  $iUpload->removed_mark() );
     $iReposUpload->set( "${prefix_top}._", --$iUpload->{_count} );
     return 1;
 }
