@@ -14,7 +14,7 @@
 # *  limitations under the License.
 # */
 #
-# $Id: Subject.pm 665 2007-05-19 13:39:00Z hikarin $
+# $Id: Subject.pm 802 2007-06-10 12:47:54Z hikarin $
 #
 
 package Zeromin::Subject;
@@ -65,7 +65,7 @@ sub update {
             $key = $1;
             $zThread->set_key($key);
             my $res_count = $zThread->load();
-            my ( undef, $count ) = $zSubject->get($key);
+            my ( undef, $count ) = @{ $zSubject->get($key) };
             my $subject = $zThread->get_subject();
             if ($count) {
                 _count( $zThread, \$res_count );
@@ -86,8 +86,8 @@ sub update {
             }
             $done{$key} = 1;
         }
-        for my $key (@{ $zSubject->to_array() }) {
-            if (!$done{$key}) {
+        for my $key ( @{ $zSubject->to_array() } ) {
+            if ( !$done{$key} ) {
                 $zSubject->remove($key);
                 push @{ $info->{removed} }, $key;
             }
@@ -189,8 +189,9 @@ sub include_special_threads { $_[0]->{__include_special} = 1 }
 sub exclude_special_threads { $_[0]->{__include_special} = 0 }
 
 sub to_array {
-    my ( $zSubject ) = @_;
-    return $zSubject->{__include_special} ? $zSubject->{_order}
+    my ($zSubject) = @_;
+    return $zSubject->{__include_special}
+        ? $zSubject->{_order}
         : [ grep { !/\A924/xms } @{ $zSubject->{_order} } ];
 }
 
